%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function doall=pacing_glacial_cycles
% Run a simple model of glacial cycles in the presence of
% milankovitch forcing and noise, and examin the resulting time series.
close all; clear all; 

%rand('state',sum(100*clock))

% define model parameters:
[V_min,V_max,V_0,a_si_on,p0,k,S,S_M,kyr,Sv,delta_t,nprint,my_colors,R,sqrt_R,observations,A_T0,A_T1,A_T2] ...
    =set_parameters;

% run the model N_ic times:
[run,n,T,time_kyr,timeseries,Milankovitch,Insolation_timeseries,N_ic,my_colors,V_max,temperature,use_milankovitch,noise_amplitude]...
    =run_model(V_min,V_max,V_0,a_si_on,p0,k,S,S_M,kyr,Sv,delta_t,nprint,my_colors,R,sqrt_R,A_T0,A_T1,A_T2);
output_file_name_ext=set_output_file_name(use_milankovitch,noise_amplitude);


% run the rayleigh equation:
%[run,n,T,time_kyr,timeseries,Milankovitch,Insolation_timeseries,N_ic,my_colors,V_max,temperature,use_milankovitch,noise_amplitude]...
%    =run_model_rayleigh(V_min,V_max,V_0,a_si_on,p0,k,S,S_M,kyr,Sv,delta_t,nprint,my_colors,R,sqrt_R,A_T0,A_T1,A_T2);
%plot(time_kyr,timeseries);

figure

% plot timeseries of the model ice volume, insolation and observations:
plot_results(n,T,kyr,timeseries,Milankovitch,observations, ...
	     Insolation_timeseries,N_ic,time_kyr,my_colors,V_max, ...
	     temperature,output_file_name_ext)

if strcmp(use_milankovitch,'none------')==0
  figure
  plot_terminations(run,n,T,delta_t,kyr,timeseries,Milankovitch,observations, ...
		    Insolation_timeseries,N_ic,time_kyr,my_colors,V_max, ...
		    temperature,output_file_name_ext)
end

drawnow
refresh

% model parameter optimization:
%optimize_model_parameters

% write model output data to file for further analysis if needed:
output_file_name=['Output/timeseries_',output_file_name_ext,'.dat'];
fid = fopen(output_file_name,'w');
for n=1:length(timeseries(:,1))
  fprintf(fid,'%14.5g',time_kyr(n),timeseries(n,:)); fprintf(fid,'\n');
end

% calculate spectrum:
title_string='(a) model: ';fprintf(1,title_string);
calculate_spectrum(timeseries,V_0,output_file_name_ext,title_string);
title_string='(b) proxy: ';fprintf(1,title_string);
timeseries=observations(:,2); V_0=1; output_file_name_ext='specmap';
calculate_spectrum(timeseries,V_0,output_file_name_ext,title_string);

status = fclose('all');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [run,n,T,time_kyr,timeseries,Milankovitch,Insolation_timeseries,N_ic,my_colors,V_max,temperature,use_milankovitch,noise_amplitude]=...
    run_model(V_min,V_max,V_0,a_si_on,p0,k,S,S_M,kyr,Sv,delta_t,nprint,my_colors,R,sqrt_R,A_T0,A_T1,A_T2)
% Based on Yossi Ashkenazy's sea ice switch model from Ashkenazy and
% Tziperman (2004), Quaternary Sciences Reviews.

% ---------------------------------
% Need to specify these:
use_milankovitch='insolation';  % set to 'insolation', 'precession',
                                % 'obliquity-', 'pure20kyr-'
                                % 'pure40kyr-'or 'none------'.
paillard_truncation=1;          % use f(x)=0.5*(x+sqrt(4*a^2+x^2))
                                % to filter the milankovitch
                                % forcing.
use_temperature=0;              % to decide if deglaciation should begin
noise_amplitude=0.04; %0.04     % nondimensional number
N_ic=4;                        % number of different integrations with
                                % different initial conditions
T=1100.0;  % 900.0                     % Integration time in kyr.  Note that
                                % it's tricky to change this when
                                % using milankovitch forcing written
                                % by the external insolation
                                % program.
superimposed_noise_amp=0.05; %0.05   % noise to add to the final model
                                % timeseries; does not enter model
                                % dynamics, and is meant to
                                % represent noise in the proxy or
                                % measurement rather than noise
                                % that actually affects the glacial
                                % dynamics. 
phase_lock=0;                   % increases the amplitude of the
                                % Milankovtich forcing, causing all
                                % terminations to initiate at
                                % approximately the same time.

% runs for paper:
%run=1;
run=input(['which run? (1-6 for timeseries a-e in paper; 0 for' ...
	   ' standard values): ']);
fprintf(1,'run=%d \n',run);
if run==1 
  use_milankovitch='none------';
  paillard_truncation=1;
  noise_amplitude=0.0;
  N_ic=2;
  superimposed_noise_amp=0.0;
elseif run==2
  use_milankovitch='insolation';
  paillard_truncation=1;
  noise_amplitude=0.0;
  N_ic=2;
  superimposed_noise_amp=0.0;
elseif run==3
  use_milankovitch='pure40kyr-';
  paillard_truncation=0;
  noise_amplitude=0.0;
  N_ic=4;
  superimposed_noise_amp=0.0;
elseif run==4
  use_milankovitch='obliquity-';
  paillard_truncation=0;
  noise_amplitude=0.0;
  N_ic=2;
  superimposed_noise_amp=0.0;
elseif run==5
  use_milankovitch='precession';
  paillard_truncation=0;
  noise_amplitude=0.0;
  N_ic=3;
  superimposed_noise_amp=0.0;
elseif run==6
  use_milankovitch='insolation';
  paillard_truncation=1;
  noise_amplitude=0.04;
  N_ic=2;
  superimposed_noise_amp=0.05;
end
				
% ---------------------------------
% override parameters in set_parameters:
% set parameters to standard values:
X=[0.33 0.26 0.7 0.23 1.0 13.0 9.0 3.0 0.03];
if phase_lock==1;
  X(9)=0.1;
end;
% Specify model parameters for obliquity and precession forced runs
if strcmp(use_milankovitch,'obliquity-'),
  X=[0.3400    0.2200    0.9100    0.1700    1.1000   14.0000    8.5000    0.2100    0.0230];
elseif strcmp(use_milankovitch,'precession'),
  X=[ 0.1600    0.2800    1.3000    0.2400    0.7400   10.0000    8.1000    0.6200    0.0200];
end;
a_si_on=X(1);p0=X(2)*Sv;k=X(3)/(40*kyr);
S=X(4)*Sv;V_max=X(5)*V_0;A_T0=X(6);
A_T1=X(7);A_T2=X(8);S_M=X(9)*Sv;
% ---------------------------------

[Insolation_timeseries,Milankovitch]...
    =set_insolation_forcing(T,use_milankovitch,paillard_truncation);


% convert integration time to model units:
T=T*kyr;

% misc initializations:
% printing frequency is every 1 time units (1kyr, expected by
% spectral routines etc):
nprint=ceil(1/delta_t); % round upward, to prevent nprint=0
% number of time steps:
N=floor(T/delta_t)+1;
% number of elements in output timeseries:
NP=length([1:nprint:N]);
timeseries=zeros(NP,N_ic);
superimposed_noise=zeros(N,1);
temperature=zeros(N,1);

% loop over different initial conditions:
for i_ic=1:N_ic
  if N_ic==1
    V_initial(i_ic)=(V_max+V_min)*0.5;
  else
    V_initial(i_ic)=V_max*0.1+(i_ic-1)*(0.9*V_max-V_max*0.1)/(N_ic-1);       % Initial value for ice volume
  end

  % V_initial(i_ic)=V_min+(i_ic-1)/(N_ic-1)*(V_max-V_min); 

  
  % use initial conditions specified above:
  xp=V_initial(i_ic);
  a_si=0;
  noise=0;
  noise_superimposed=0;

  % Integrate:
  for n=1:1:N
    time(n)=-T+(n-1)*delta_t;
    Insolation=Insolation_timeseries(T/kyr+1+round(time(n)/kyr));
    plot_insolation(n)=Insolation;
    
    V(n)=xp;

    % noise that enters the equation:
    rand('state',n*i_ic);
    noise=R*noise+sqrt_R*2*(rand(1)-0.5);
    plot_noise(n)=noise;
    % additive noise added to the final model time series:
    rand('state',n*i_ic+7);
    noise_superimposed=R*noise_superimposed+sqrt_R*2*(rand(1)-0.5);
    superimposed_noise(n)=noise_superimposed*superimposed_noise_amp;

    % calculate a global ocean summer temperature based on land ice
    % albedo and summer insolation:
    temperature(n)=A_T0-A_T1*V(n)/V_0+A_T2*Insolation;
    
    % decide if to turn sea ice on or off based on ice volume (and
    % possibly "temperature" too, if this part is uncommented below):
    if use_temperature
      if xp>V_max || temperature(n)<0
	a_si=a_si_on;
      elseif xp<V_min
	a_si=0;
      end
    else
      if xp>V_max
	a_si=a_si_on;
      elseif xp<V_min
	a_si=0;
      end
    end
    x_tilde=xp+delta_t*  rhs(xp,     a_si,Insolation,V_min,V_max,V_0,p0,k,S,S_M,noise,noise_amplitude);
    xp=     xp+delta_t*( rhs(xp,     a_si,Insolation,V_min,V_max,V_0,p0,k,S,S_M,noise,noise_amplitude) ...
			 + rhs(x_tilde,a_si,Insolation,V_min,V_max,V_0,p0,k,S,S_M,noise,noise_amplitude) ...
			 )*0.5;
  end

  
  if noise_amplitude~=0
    %figure(100); clf; hold on;
    %plot(S*noise_amplitude*plot_noise,'k');
    %plot(S_M*plot_insolation,'k');
    S2N(i_ic)=var(S_M*plot_insolation)/var(S*noise_amplitude*plot_noise);
    %title(num2str(S2N));
    %drawnow
  else
    S2N(i_ic)=0;
  end
  if i_ic==N_ic
    fprintf(1,'mean signal to noise=%14.4g \n',mean(S2N));
  end
  
  timeseries(1:NP,i_ic)=V(1:nprint:N)'+V_0*superimposed_noise(1:nprint:N);
end
% The following is needed to set one of the output variables for
% this function:
time_kyr=time(1:nprint:N)/kyr;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [run,NP,T,time_kyr,timeseries,Milankovitch,Insolation_timeseries,N_ic,my_colors,V_max,temperature,use_milankovitch,noise_amplitude]=...
    run_model_rayleigh(V_min,V_max,V_0,a_si_on,p0,k,S,S_M,kyr,Sv,delta_t,nprint,my_colors,R,sqrt_R,A_T0,A_T1,A_T2)
% modified rayleigh equation

% ---------------------------------
% Need to specify these:
use_milankovitch='insolation';  % set to 'insolation', 'precession',
                                % 'obliquity-', 'pure20kyr-'
                                % 'pure40kry-'or 'none------'.
paillard_truncation=1;          % use f(x)=0.5*(x+sqrt(4*a^2+x^2))
                                % to filter the milankovitch
                                % forcing.
noise_amplitude=0.0;           % nondimensional number
N_ic=2;                        % number of different integrations with
                                % different initial conditions
T=2000.0;                       % Integration time in kyr.  Note that
                                % it's tricky to change this when
                                % using milankovitch forcing written
                                % by the external insolation
                                % program.
superimposed_noise_amp=0.0;   % noise to add to the final model
                                % timeseries; does not enter model
                                % dynamics, and is meant to
                                % represent noise in the proxy or
                                % measurement rather than noise
                                % that actually affects the glacial
                                % dynamics. 
% ---------------------------------

[Insolation_timeseries,Milankovitch]...
    =set_insolation_forcing(T,use_milankovitch,paillard_truncation);

% convert integration time to model units:
T=T;

% misc initializations:
% time step:
delta_t=0.02;
% printing frequency is every 1 time units (1kyr, expected by
% spectral routines etc):
nprint=round(1/delta_t); 
% number of time steps:
N=floor(T/delta_t)+1; 
% number of elements in output timeseries:
NP=floor(N/nprint)+1;
timeseries=zeros(NP,N_ic);
superimposed_noise=zeros(N,1);

% loop over different initial conditions:
V_max=10;V_min=-100;
for i_ic=1:N_ic
  if N_ic==1
    V_initial(i_ic)=(V_max+V_min)*0.5;
  else
    V_initial(i_ic)=V_min+(i_ic-1)*(V_max-V_min)/(N_ic-1);   % Initial value for ice volume
  end
  
  % use initial conditions specified above:
  xp=[V_initial(i_ic) 0];
  noise=0;
  noise_superimposed=0;

  % Integrate:
  for n=1:1:N
    time(n)=-T+(n-1)*delta_t;
    Insolation=Insolation_timeseries(T+1+round(time(n)));
    V(n)=xp(1); % in the case of the rayleigh equation, xp is a 2-vector

    % noise that enters the equation:
    rand('state',n*i_ic);
    noise=R*noise+sqrt_R*2*(rand(1)-0.5);
    plot_noise(n)=noise;
    % additive noise added to the final model time series:
    rand('state',n*i_ic+7);
    noise_superimposed=R*noise_superimposed+sqrt_R*2*(rand(1)-0.5);
    superimposed_noise(n)=noise_superimposed*superimposed_noise_amp;
    
    x_tilde=xp+delta_t*  rhs_rayleigh(xp,Insolation);
    xp=     xp+delta_t*( rhs_rayleigh(xp,Insolation) ...
			 + rhs_rayleigh(x_tilde,Insolation) ...
			 )*0.5;
  end
  
  timeseries(1:NP,i_ic)=V(1:nprint:N)'+superimposed_noise(1:nprint:N);
end
% The following is needed to set one of the output variables for
% this function:
time_kyr=time(1:nprint:N);

% not used, but needs to be returned as output for consistency with
% run_model

temperature=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_results(n,T,kyr,timeseries,Milankovitch,observations, ...
		      Insolation_timeseries,N_ic,time_kyr, ...
		      my_colors,V_max,temperature,output_file_name_ext)

% Normalize time series of ice volume to [0,1] and plot:
max1=max(max(timeseries(:,:)));
min1=min(min(timeseries(:,:)));
for i_ic=1:N_ic
  for i=1:length(timeseries(:,i_ic))
    timeseries(i,i_ic)=(timeseries(i,i_ic)-min1)/(max1-min1);
  end
end

% plot observations time series:
% -------------------------
if 1
  h=plot(-observations(:,1),observations(:,2));
  set(h,'linewidth',3);
  set(h,'color',[0.7 0.7 0.7]);  % larger numbers give lighter gray
  hold on
end


% plot model output for ice volume time series:
% ---------------------------------------------
for i_ic=1:N_ic
  % Plot time series of insolation forcing:
  b=length(Milankovitch(:,1))-round(T/kyr);
  e=length(Milankovitch(:,1));
  plot(Milankovitch(b:e,1)/1000,Insolation_timeseries(:)/6-0.3,'b');
  %plot(time_kyr(1:n),plot_noise(1:n),'g');
  hold on
  plot(time_kyr(1:n),timeseries(1:n,i_ic),my_colors(1+mod(i_ic,length(my_colors(:,1))-1),:))
  h=xlabel('time (kyr)');
  set(h,'FontName','Helvetica','FontWeight','bold','FontSize',12);
  h=ylabel('65N July insolation                     ice volume');
  set(h,'FontName','Helvetica','FontWeight','bold','FontSize',12);
  % plot(time_kyr(1:n),temperature(1:n)/15+1,'b')
  grid
end

% plot vertical lines from minima of the insolation forcing:
% ----------------------------------------------------------
V=axis;
%axis([-700 0 V(3) V(4)]);
for i=2:length(Insolation_timeseries(:))-1
  % is this a minimum in insolation:
  if Insolation_timeseries(i)<Insolation_timeseries(i-1)&&...
	Insolation_timeseries(i)<Insolation_timeseries(i+1)
    t_peak=Milankovitch(i,1)/1000;
%    plot([t_peak,t_peak],[Insolation_timeseries(i)/6-0.3,V(4)],'r:')
  end
end

% some fine tunning of the final figure:
set(gca,'YTick',[0.0 0.2 0.4 0.6 0.8 1.0]);
h=gca;
set(h,'FontName','Helvetica','FontWeight','bold','FontSize',12);

% print and save the figure:
print(gcf,'-depsc',['Output/timeseries_',output_file_name_ext,'.eps'])
saveas(gcf,['Output/timeseries_',output_file_name_ext,'.fig'],'fig')


% print and save the figure:
axis([-800 0 0.0 1.1]);
ylabel('ice volume')
print -depsc Output/timeseries-fit2specmap.eps
saveas(gcf,'Output/timeseries-fit2specmap.fig','fig')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_terminations(run,n,T,delta_t,kyr,timeseries,Milankovitch ...
			   ,observations,Insolation_timeseries,N_ic,time_kyr, ...
			   my_colors,V_max,temperature,output_file_name_ext)
% plot terminations only super imposed on one another together with
% Milankovitch forcing:

axes('Position',[0.2 0.2 0.5 0.6])

% Normalize time series of ice volume to [0,1] and plot:
for i_ic=1:N_ic
  max1=max(timeseries(:,i_ic));
  min1=min(timeseries(:,i_ic));
end
timeseries(:,:)=(timeseries(:,:)-min1)/(max1-min1);

% find termination times:
% manually:
%termination_starts_manual=[-798,-633,-430,-251,-142,-27];
% automatically:
local_max_range=45;
itermination=1;
maximum=1;
for i=1:length(timeseries(:,1))
  this_is_a_maximum=1;
  minimum=1.e10;
  for j=max(1,i-local_max_range):min(i+local_max_range,length(timeseries(:,1)))
    if timeseries(i,1)<timeseries(j,1) || timeseries(i,1)<0.8;
      this_is_a_maximum=0;
      maximum=timeseries(i,1);
    end
    minimum=min(minimum,timeseries(j,1));
  end
  % this is a maximum ice volume that occurs just before
  % termination, if three conditions are satisfied: it is a local
  % maximum over the window defined by local_max_range; the time is
  % during the past 800kyr; the maximum is significantly larger
  % than the minimum within the same window:
  if this_is_a_maximum && time_kyr(i)>-800 && maximum/minimum>2
    termination_starts(itermination)=i;
    itermination=itermination+1;
  end
end
termination_starts=termination_starts-round(T/kyr)-1;
window_width=15;
window_left=round(2*window_width/3)+2;
window_right=round(4*window_width/3)+2;
for it=1:length(termination_starts)
  termination_times(it,1)=termination_starts(it)-window_left;
  termination_times(it,2)=min(termination_starts(it)+window_right,0);
end

% loop over all terminations, plot them:
for itermination=1:length(termination_times(:,1))
  % set begining and end points of current termination; units are not
  % years, but index in the array Milankovitch();
  % The following assumes that the resolution of the Milankovitch
  % forcing is 1kyr.  This is probably assumed in other places in
  % this code as well...:
  begt=termination_times(itermination,1)+length(Milankovitch(:,1));
  endt=termination_times(itermination,2)+length(Milankovitch(:,1));
  if endt>length(Insolation_timeseries)-1,  %PH, ensure that
                                            %Insolation_timeseries(i+1)
                                            %is within bounds
    endt=length(Insolation_timeseries)-1;
  end;
  
  % set mid point of current termination, the time units are kyrs:
  midt_kyr=round((begt+endt)/2)-length(Milankovitch(:,1));
  
  % plot observations time series:
  % -------------------------
  if 0
    h=plot(-observations(begt:endt,1),observations(begt:endt,2));
    set(h,'linewidth',3);
    set(h,'color',[0.7 0.7 0.7]);  % larger numbers give lighter gray
    hold on
  end

  % plot model output for ice volume time series:
  % ---------------------------------------------
  for i_ic=1
    % Plot time series of insolation forcing:
    plot(Milankovitch(begt:endt,1)/1000-midt_kyr ...
	 ,Insolation_timeseries(begt:endt)/6-0.5 ...
	 ,my_colors(1+mod(itermination,length(my_colors(:,1))-1),:));
    %plot(time_kyr(1:n),plot_noise(1:n),'g');
    hold on
    plot(time_kyr(begt:endt)-midt_kyr,timeseries(begt:endt,i_ic) ...
	 ,my_colors(1+mod(itermination,length(my_colors(:,1))-1),:))
    h(1)=xlabel('time (kyr)');
    % put spaces before ice volume ylabel to have it printed upward
    % on the axis:
    h(2)=ylabel('65N July insolation               ice volume');
    if run==2
      h(3)=title('(a) Model terminations & Milankovitch forcing');
    elseif run==3
      h(3)=title('(b) Model terminations & Insolation forcing');
    end
    set(h,'FontName','Helvetica','FontWeight','bold','FontSize',12);
    % plot(time_kyr(1:n),temperature(1:n)/15+1,'b')
    %grid
  end
  
  % plot vertical lines from minima of the insolation forcing:
  % ----------------------------------------------------------
  V=axis;
  for i=begt:endt
    % is this a minimum in insolation:
    if Insolation_timeseries(i)<Insolation_timeseries(i-1)&&...
	  Insolation_timeseries(i)<Insolation_timeseries(i+1)
      t_peak=Milankovitch(i,1)/1000-midt_kyr;
      plot([t_peak,t_peak],[Insolation_timeseries(i)/6-0.5,V(4)] ...
	   ,[my_colors(1+mod(itermination,length(my_colors(:,1))-1),1) ':']);
      % ,'r:');
      % ,my_colors(1+mod(itermination,length(my_colors(:,1))-1),:));
    end
  end
end

% some fine tunning of the final figure:
% ([XMIN XMAX YMIN YMAX]):
V=axis; axis([-window_width window_width V(3) 1.1])
set(gca,'YTick',[0.0 0.2 0.4 0.6 0.8 1.0]);
h=gca; set(h,'FontName','Helvetica','FontWeight','bold','FontSize',11);

% print and save the figure:
print(gcf,'-depsc',['Output/terminations_',output_file_name_ext,'.eps'])
saveas(gcf,['Output/terminations_',output_file_name_ext,'.fig'],'fig')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ff=rhs(x,a_si,Insolation,V_min,V_max,V_0,p0,k,S,S_M,noise,noise_amplitude)
% return the rhs of the dynamical system:

V=x;

% rhs of ice volume equation:
ff(1)=(p0-k*V)*(1-a_si)-S*(1+noise_amplitude*noise)-S_M*Insolation;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ff=rhs_rayleigh(x,Insolation)
% return the rhs of the modified rayleigh oscillator:

mu=1.0; % larger value leads to longer periods
alpha=4.0;
f1=-2.0;  % insolation forcing amplitude
f2=10.0;  % insolation forcing amplitude
omega2=(2*pi/100)^2

ff(1)=x(2)+f1*Insolation;
ff(2)=mu*(1-alpha*x(2)-x(2)^2)*x(2)-omega2*x(1)+f2*Insolation;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [V_min,V_max,V_0,a_si_on,p0,k,S,S_M,kyr,Sv,delta_t,nprint,my_colors,R,sqrt_R,observations,A_T0,A_T1,A_T2] ...
    =set_parameters;
% set glacial model parameters

% define model units:
meter=1;
second=1;
Sv=1.e6*meter^3/second;
km=1000*meter;
year=3600*24*365*second;
kyr=1000*year;

% specify model parameters: 
V_0=45.0e6*km^3;
V_min=3.0e6*km^3;
% [a_si_on,p0/Sv,k*40kyr,S/Sv,V_max/V_0,S_M/Sv,A_T0,A_T1,A_T2]=0.46 0.26 0.7 0.23 1.0 13.0 9.0
a_si_on=0.46;
p0=0.26*Sv;
k=0.70/(40*kyr);
S=0.23*Sv;
V_max=1.0*V_0;
S_M=0.03*Sv; % 0.03
A_T0=13;
A_T1=9;
A_T2=3;

% time step: needs to be small enough for a given problem:
delta_t=1*kyr;
% Print (and save) every nprint steps:
nprint=1;

% colors of line plots:
my_colors=['b- ';'g- ';'r- ';'m- ';'k- ';'c- ';'b--';'g--';'r--';'k--';'m--';'c--'];

% prepare coeffs for markov process:
T_corr=3*kyr;
R=exp(-delta_t/T_corr);
sqrt_R=sqrt(1-R^2);

% get the observations:
load Input/observations.dat;

% normalize data to [0,1]:
spec_max=max(observations(:,2));
spec_min=min(observations(:,2));
observations(:,2)=(observations(:,2)-spec_min)/(spec_max-spec_min);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Insolation_timeseries,Milankovitch]...
    =set_insolation_forcing(T,use_milankovitch,paillard_truncation)
% choose and set Milankovitch forcing

% Load Milankovitch forcing: this is the renamed output file from the
% program ~/etc/Misc.f/Insolation-Milankovitch/insolation-eli-4pacing.f:
load Input/Milankovitch.dat;
b=length(Milankovitch(:,1))-round(T);
e=length(Milankovitch(:,1));

% trancate Milankovitch forcing array only to the period of the
% model run:
tmpM(:,:)=Milankovitch(b:e,:); clear Milankovitch; Milankovitch=tmpM;

% output file from insolation-eli-4pacing.f contains: 
% time, latitude, insolation, eccentricity, precession, longitude, obliquity
if strcmp(use_milankovitch,'insolation')
  Insolation_timeseries=Milankovitch(:,3);
elseif strcmp(use_milankovitch,'precession')
  Insolation_timeseries=Milankovitch(:,5);
elseif strcmp(use_milankovitch,'obliquity-')
  Insolation_timeseries=Milankovitch(:,7);
elseif strcmp(use_milankovitch,'none------')
  Insolation_timeseries=0*Milankovitch(:,3);
elseif strcmp(use_milankovitch,'pure20kyr-')
  for i=1:length(Milankovitch(:,3))
    % Insolation_timeseries has a data point every 1000 years:
    Insolation_timeseries(i)=sin(i*2*pi/20);
  end
elseif strcmp(use_milankovitch,'pure40kyr-')
  for i=1:length(Milankovitch(:,3))
    % Insolation_timeseries has a data point every 1000 years:
    Insolation_timeseries(i)=sin(i*2*pi/40);
  end
else
  disp(' *** Error: wrong value for use_milankovitch ***');
  return;
end

% normalize insolation forcing amplitude and eliminate its mean: 
if strcmp(use_milankovitch,'none------')~=1
  avg=mean(Insolation_timeseries(:));
  Insolation_timeseries(:)=Insolation_timeseries(:)-avg;
  rms=std(Insolation_timeseries(:));
  Insolation_timeseries(:)=Insolation_timeseries(:)/rms;

  % use truncation of milankovitch forcing, a-la Paillard (1998):
  if paillard_truncation
    % disp('using paillard truncation.');
    for i=1:length(Insolation_timeseries)
      x=Insolation_timeseries(i);
      a=1;
      Insolation_timeseries(i)=0.5*(x+sqrt(4*a^2+x^2));
    end
    % need to renormalize now:
    avg=mean(Insolation_timeseries(:));
    Insolation_timeseries(:)=Insolation_timeseries(:)-avg;
    rms=std(Insolation_timeseries(:));
    Insolation_timeseries(:)=Insolation_timeseries(:)/rms;
  end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function optimize_model_parameters
% optimize parameters to fit observations:

% define model parameters:
[V_min,V_max,V_0,a_si_on,p0,k,S,S_M,kyr,Sv,delta_t,nprint,my_colors,R,sqrt_R,observations,A_T0,A_T1,A_T2] ...
    =set_parameters;

% run optimization:
options = optimset; % 'TolFun',0.001,'TolX',1.0e3);
X0=[a_si_on,p0/Sv,k*(40*kyr),S/Sv,V_max/V_0,A_T0,A_T1,A_T2];%,S_M/Sv];
fprintf(1,'init  [a_si_on,p0/Sv,k*40kyr,S/Sv,V_max/V_0,S_M/Sv,A_T0,A_T1,A_T2]=%8.5f %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f \n%8.5f\n',X0)
X = fminsearch(@fit2observations,X0,options);
% print parameters before and after optimization:
fprintf(1,'init  [a_si_on,p0/Sv,k*40kyr,S/Sv,V_max/V_0,S_M/Sv,A_T0,A_T1,A_T2]=%8.5f %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f \n%8.5f\n',X0)
fprintf(1,'final [a_si_on,p0/Sv,k*40kyr,S/Sv,V_max/V_0,S_M/Sv,A_T0,A_T1,A_T2]=%8.5f %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f \n%8.5f\n',X)

% set parameters to optimized values:
a_si_on=X(1);p0=X(2)*Sv;k=X(3)/(40*kyr);S=X(4)*Sv;V_max=X(5)*V_0;A_T0=X(6);A_T1=X(7);A_T2=X(8);%S_M=X(9)*Sv;
% override optimization if desired:
%S_M=0.02*Sv;

% run the model once more with optimized values:
[run,n,T,time_kyr,timeseries,Milankovitch,Insolation_timeseries,N_ic,my_colors,V_max,temperature,use_milankovitch,noise_amplitude]...
    =run_model(V_min,V_max,V_0,a_si_on,p0,k,S,S_M,kyr,Sv,delta_t,nprint,my_colors,R,sqrt_R,A_T0,A_T1,A_T2);

% plot optimized model results:
figure
plot_results(n,T,kyr,timeseries,Milankovitch,observations, ...
	     Insolation_timeseries,N_ic,time_kyr,my_colors,V_max,temperature)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 function cost=fit2observations(X)
% fit model output to observations timeseries:

% define model parameters:
[V_min,V_max,V_0,a_si_on,p0,k,S,S_M,kyr,Sv,delta_t,nprint,my_colors,R,sqrt_R,observations,A_T0,A_T1,A_T2] ...
    =set_parameters;

if delta_t ~= kyr
  display('time step must be 1kyr for optimization')
  quite
end

% set parameters that are being optimized:
a_si_on=X(1);p0=X(2)*Sv;k=X(3)/(40*kyr);S=X(4)*Sv;V_max=X(5)*V_0;A_T0=X(6);A_T1=X(7);A_T2=X(8);%S_M=X(9)*Sv;
% override optimization if desired:
%S_M=0.02*Sv;

% run the model once: (must have N_ic = 1):
[run,n,T,time_kyr,timeseries,Milankovitch,Insolation_timeseries,N_ic,my_colors,V_max,temperature,use_milankovitch,noise_amplitude]...
    =run_model(V_min,V_max,V_0,a_si_on,p0,k,S,S_M,kyr,Sv,delta_t,nprint,my_colors,R,sqrt_R,A_T0,A_T1,A_T2);
if N_ic~=1; display('Must set N_ic = 1 for optimization.'); quit; end;

% set the start and length of the segment of the time series to be
% optimized (kyr BP and kyr, respectively):
start_optimize=300;
length_to_optimize=120;
beg1=length(timeseries(:,1))-start_optimize;
end1=length(timeseries(:,1))-start_optimize+length_to_optimize;
beg2=length(observations(:,2))-start_optimize;
end2=length(observations(:,2))-start_optimize+length_to_optimize;;
% reverse observations to arrange in chronological order:
length_observations=length(observations(:,2));
for i=1:length_observations
  observations_tmp(i,:)=observations(length_observations-i+1,:);
end
observations=observations_tmp;

% A straight forward least square cost measuring the distance
% between the curves:
if 1
  cost=0;
  for i=1:length_to_optimize
    cost=cost+(timeseries(beg1+i-1,1)/V_0-observations(beg2+i-1,2))^2;
  end
  cost=cost/(end1-beg1);
end

% simple least square, applied at a set of time points:
% the following options are for optimizing over a set of
% time points specified in the array times_to_optimize;
times_to_optimize=[435,405,342,330,249,238,135,122,19,1];
itime_to_optimize=1;
if 0
  cost=0;
  for i=1:length_to_optimize
    %fprintf(1,['itime_to_optimize=%d,beg2+i-1=%d, observations(beg2+i-1,1)=%d,times_to_optimize(itime_to_optimize)=%d\n'],itime_to_optimize,beg2+i-1,observations(beg2+i-1,1),times_to_optimize(itime_to_optimize));
    if itime_to_optimize<=length(times_to_optimize)
      if observations(beg2+i-1,1)==times_to_optimize(itime_to_optimize)
	cost=cost+abs(timeseries(beg1+i-1,1)/V_0-observations(beg2+i-1,2))^1;
	itime_to_optimize=itime_to_optimize+1;
      end
    end
  end
  cost=cost/(end1-beg1);
end

% An alternative cost calculation based on the minimum distance
% between the two curves at each point.  That is, distance is measured
% not vertically, but in the direction that results in the smallest
% value:
if 0
  cost=0;
  for i=1:length_to_optimize
    % For each point [i] in the timeseries of model results, search for
    % the point [i+j] in the observations that has the smallers distance
    % to timeseries(i).  First, set the limits on [j]:
    jstart=max(-20,i-beg2);
    jend=min(20,end2-(beg2+i-1));
    % Normalization of the horizontal distance between the two time
    % series:
    delta_j=15;
    distance_min=1.0e10;
    for j=jstart:jend
      distance=(timeseries(beg1+i-1,1)/V_max-observations(beg2+i-1+j,2))^2+j^2/delta_j^2;
      if distance<distance_min; j_min=j; distance_min=distance; end
      % fprintf(1,'j,distance=%d %12.4g\n',j,distance)
    end
    % fprintf(1,'j_min,distance_min=%d %12.4g\n',j_min,distance_min)
    cost=cost+distance_min;
  end
  cost=cost/(end1-beg1);
end

% set high cost value if upper or lower bounds are violated:
if 0
  Lower_bound=[0.1 0.10 0.1 0.2 0.0  0.9];
  Upper_bound=[0.9 0.40 3.0 0.3 0.08 1.1];
  violated=0;
  for i=1:length(X)
    if X(i)<Lower_bound(i)||X(i)>Upper_bound(i); violated=1; end
  end
  
  if violated
    fprintf(1,'bounds violated, X=%g %g %g %g %g %g; ',X);
    cost=1.e6;
  end
end

fprintf(1,'cost=%12.4g\n',cost)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function [P,s,ci] = pmtmPH(x,dt,nw,qplot,nfft);
%
%Computes the power spectrum using the multi-taper method with adaptive weighting.
%
% Inputs:
%   x      - Input data vector.
%   dt     - Sampling interval, default is 1.
%   nw     - Time bandwidth product, acceptable values are
%            0:.5:length(x)/2-1, default is 3.  2*nw-1 dpss tapers
%            are applied except if nw=0 a boxcar window is applied 
%            and if nw=.5 (or 1) a single dpss taper is applied.
%   qplot  - Generate a plot: 1 for yes, else no.  
%   nfft   - Number of frequencies to evaluate P at, default is
%            length(x) for the two-sided transform. 
%
% Outputs:
%   P      - Power spectrum computed via the multi-taper method.
%   s      - Frequency vector.
%   ci     - 95% confidence intervals. Note that both the degrees of freedom
%            calculated by pmtm.m and chi2conf.m, which pmtm.m calls, are
%            incorrect.  Here a quick approximation method is used to
%            determine the chi-squared 95% confidence limits for v degrees
%            of freedom.  The degrees of freedom are close to but no larger
%            than (2*nw-1)*2; if the degrees of freedom are greater than
%            roughly 30, the chi-squared distribution is close to Gaussian.
%
%            The vertical ticks at the top of the plot indicate the size of
%            the full band-width.  The distance between ticks would remain
%            fixed in a linear plot.  For an accurate spectral estimate,
%            the true spectra should not vary abruptly on scales less than
%            the full-bandwidth.
%
%Other toolbox functions called: dpps.m; and if nfft does not equal length(x), czt.m

function [P,s,ci] = pmtmPH(x,dt,nw,qplot,nfft);

nw=2;
if nargin<1, %make red-noise with a periodic component.
  x=0; 
  for ct=2:500,
      x(ct)=.8*x(ct-1)+randn; 
  end;
  x=x/std(x)+sqrt(.5)*sin(2*pi*[1:500]/41+rand*2*pi)+sin(2*pi*[1:500]/100+rand*2*pi); 
end;

if nargin<2, dt=1; end;
if nargin<3, nw=3; end;
if nargin<4, qplot=1; end;
if nargin<5, nfft=length(x); end;
if nargin<6, cl=.95; end;

x   = x(:);
nx  = length(x);
k   = min(round(2*nw),nx); 
k   = max(k-1,1);
s   = 0:1/(nfft*dt):1/dt-1/(nfft*dt);
s   = s(:);
w   = nw/(dt*nx);       % half-bandwidth of the dpss

%Compute the discrete prolate spheroidal sequences
[E,V]=dpss(nx,nw,k);

%Compute the windowed DFTs.
if nx<=nfft
   Pk=abs(fft(E(:,1:k).*x(:,ones(1,k)),nfft)).^2;
else  %compute DFT on nfft evenly spaced samples around unit circle:
   Pk=abs(czt(E(:,1:k).*x(:,ones(1,k)),nfft)).^2; 
end

%Iteration to determine adaptive weights:    
   if k>1,
   sig2 = x'*x/nx;             % power
   P    = (Pk(:,1)+Pk(:,2))/2;   % initial spectrum estimate
   Ptemp= zeros(nfft,1);
   P1   = zeros(nfft,1);
   tol  = .0005*sig2/nfft;     % usually within 'tol'erance in about three iterations, see equations from [2] (P&W pp 368-370).   
   a    = sig2*(1-V);
   while sum(abs(P-P1)/nfft)>tol            
      b=(P*ones(1,k))./(P*V'+ones(nfft,1)*a'); % weights
      wk=(b.^2).*(ones(nfft,1)*V');            % new spectral estimate
      P1=(sum(wk'.*Pk')./ sum(wk'))';
      Ptemp=P1; P1=P; P=Ptemp;                 % swap P and P1
   end
   %Determine equivalent degrees of freedom, see p.  of Percival and Walden 1993.
   v=(2*sum((b.^2).*(ones(nfft,1)*V'),2).^2)./sum((b.^4).*(ones(nfft,1)*V.^2'),2);
   else, %simply the periodogram;
     P=Pk;
     v=2*ones(nfft,1);
   end;

%cut records
select=1:(nfft+1)/2+1;
P=P(select); 
s=s(select);
v=v(select);

%Chi-squared 95% confidence interval
%approximation from Chamber's et al 1983; see Percival and Walden p.256, 1993
ci(:,1)=1./(1-2./(9*v)-1.96*sqrt(2./(9*v))).^3;
ci(:,2)=1./(1-2./(9*v)+1.96*sqrt(2./(9*v))).^3;

if qplot==1,
%ian figure(1); clf;
hold on;
cu=P(2:end).*ci(2:end,1);
cl=P(2:end).*ci(2:end,2);
%confidence interval
c=[.92 .92 .92];
h=fill([s(2); s(2:end); flipud([s(2:end); s(end)])],[cu(1); cl; flipud([cu; cl(end)])],c);
set(h,'edgecolor',c);
h=plot(s,P); set(h,'linewidth',2);
%bandwidth indicators
axis tight; h=axis; 
plot([s(2) h(2)],[h(4) h(4)],'k');
for ds=min(s):2*w:max(s);
  plot([ds ds],[h(4)/1.3 h(4)],'k');
end;
set(gca,'xscale','log'); set(gca,'yscale','log');
xlabel('frequency (cycles/deltat)')
ylabel('power density (units^2/cycle/deltat)');
end;

%-------------------------------------------------------Other Possibilities
%Can use stats toolbox chi2inv for a more accurate approx.
%if exist('chi2inv')>0,  % need the stats toolbox for chi2inv
%ci=2*k./[chi2inv((1-cl)/2,2*k) chi2inv(1-(1-cl)/2,2*k)]; % confidence intervals

%Code for non-adaptive weightings based on the eigen-value of each dpss 
%P = Pk*V/k;

%plot confidence interval for eigen-weighting
%axis tight; h=axis;
%x=(50*h(1)+h(2))/51;
%y=(100*h(3)+h(4))/101;
%plot([x x],y.*ci,'k');
%plot([x-w x+w],[y y],'k');
%plot(x,y,'k.');
%axis tight;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function calculate_spectrum(timeseries,V_0,output_file_name_ext,title_string);
% use Peter's routine to calculate and plot the spectrum.

% normalize timeseries:
x=timeseries(:,1)/V_0;
avg=mean(x(:)); x(:)=x(:)-avg;
rms=std(x(:)); x(:)=x(:)/rms;

% set parameters and calculate spectrum:
dt=1;nw=3;qplot=0;if qplot;figure;end;nfft=length(x);
[P,s,ci] = pmtmPH(x,dt,nw,qplot,nfft);

% calculate upper and lower confidence intervals:
Ns=length(s);
cu(2:Ns)=P(2:Ns).*ci(2:Ns,1);
cl(2:Ns)=P(2:Ns).*ci(2:Ns,2);
period(1)=0;period(2:Ns)=1./s(2:Ns);

figure
gray=[.8 .8 .8];
axes('Position',[0.2 0.2 0.4 0.4])

% calculate the intergrated power:
accumulated_power(1)=P(1)*(s(2)-s(1));
power_100kyr=0;
power_precession17to25=0;
power_obliquity37to45=0;
for i=2:length(s)-1
  Delta_accumulated_power=P(i)*(s(i+1)-s(i-1));
  accumulated_power(i)=accumulated_power(i-1)+Delta_accumulated_power;
  if period(i)>92;
    power_100kyr=power_100kyr+Delta_accumulated_power;
  end
  if period(i)>=17 && period(i)<=25
    power_precession17to25=power_precession17to25+Delta_accumulated_power;
  end
  if period(i)>=37 && period(i)<=45
    power_obliquity37to45=power_obliquity37to45+Delta_accumulated_power;
  end
end

% draw the confidence intervals as a gray background:
h=fill([period(2:Ns),period(Ns:-1:2),period(2)] ...
      ,[cu(2:Ns),    cl(Ns:-1:2),    cu(2)],gray);
set(h,'edgecolor',gray);
set(gca,'xscale','log'); set(gca,'yscale','log');
hold on
% draw the spectrum as a black line:
loglog(period(2:Ns),P(2:Ns),'k');

% reverse x axis to go from long periods to shorter ones:
set(gca,'XDir','reverse','YDir','default') 

% find minimum and maximum values of spectrum in plotted range:
period_min=2; period_max=500;
max1=-1.e10;min1=1.e10;
for i=1:Ns
  if period(i)>period_min&&period(i)<period_max
    if cu(i)>max1; max1=cu(i); end
    if cl(i)<min1; min1=cl(i); end
  end
end

% add arrows indicating Milankovitch frequencies:
% close all; clear all; open Output/spectrum.fig;
h(1)=line([100 100],[0.5 0.5]);       h(2)=line([100 100],[0.5/20 0.5]); 
h(3)=line([41 41],  [0.05 0.05]);     h(4)=line([41 41],  [0.05/20 0.05]); 
h(5)=line([23 23],  [0.006 0.006]);   h(6)=line([23 23],  [0.006/20 0.006]); 
h(7)=line([19 19],  [0.004 0.004]); h(8)=line([19 19],  [0.004/20 0.004]); 
for i=1:2:7; set(h(i), 'Marker','^','MarkerSize',6,'Color','k'); end;
for i=2:2:8; set(h(i),'Color','k'); end;

% set period range to plot, print labels:
V=axis; axis([period_min period_max min1*0.5 max1*1.5])

h(1)=gca;
h(2)=xlabel('Period (kyr)');
h(3)=ylabel('Power spectrum');
h(4)=title([title_string,sprintf('prec=%d%%; obliq=%d%% 100kyr=%d%%' ...
		  ,round(100*power_precession17to25) ...
		   ,round(100*power_obliquity37to45) ...
		   ,round(100*power_100kyr))]);
for i=1:4;set(h(i),'FontName','Helvetica','FontWeight','bold','FontSize',12);end;

% draw accumulated power:
loglog(period(2:Ns-1),accumulated_power(2:Ns-1),'r--');

% tick marks
set(gca,'XTick',[0.1 1 10 100 1000]);
set(gca,'YTick',[0.001 0.01 0.1 1 10 100]);

fprintf('power at 17-25kyr=%d%%; 37-45kyr=%d%%; >92kyr=%d%%\n' ...
		  ,round(100*power_precession17to25) ...
		   ,round(100*power_obliquity37to45) ...
		   ,round(100*power_100kyr));

saveas(gcf,['Output/spectrum_',output_file_name_ext,'.fig'],'fig');
print(gcf,'-depsc',['Output/spectrum_',output_file_name_ext,'.eps'])
refresh

% $$$ x=observations(:,2);
% $$$ avg=mean(x(:)); x(:)=x(:)-avg;
% $$$ rms=std(x(:)); x(:)=x(:)/rms;
% $$$ dt=1;nw=3;qplot=1;if qplot;figure;end;nfft=length(x);
% $$$ [P,s,ci] = pmtmPH(x,dt,nw,qplot,nfft);
% $$$ title('SPECMAP');
% $$$ print(gcf,'-depsc','Output/spectrum_specmap.eps')
% $$$ refresh


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function output_file_name_ext=set_output_file_name(use_milankovitch,noise_amplitude)
% Set output file name extension based on whether we use noise and
% what kind of Milankovitch forcing:

if noise_amplitude==0; 
  noise_string='no_noise';
else
  noise_string='with_noise';
end

if     strcmp(use_milankovitch,'insolation')==1;
  milankovitch_string='65N';
elseif strcmp(use_milankovitch,'obliquity-')==1;
  milankovitch_string='obliq';
elseif strcmp(use_milankovitch,'precession')==1;
  milankovitch_string='preces';
elseif strcmp(use_milankovitch,'pure20kyr-')==1;
  milankovitch_string='20kyr';
elseif strcmp(use_milankovitch,'pure40kyr-')==1;
  milankovitch_string='40kyr';
elseif strcmp(use_milankovitch,'none------')==1;
  milankovitch_string='no_milank';
end

output_file_name_ext=[milankovitch_string,'_',noise_string];
